"""
Module realize lists mixin
"""

from typing import Awaitable, List, Optional, Union

from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload

from .base import Backport3MixinStub


class ListMixin(Backport3MixinStub):
    """List Mixin to work with lists."""

    def createList(
        self, listType: str, listData: Optional[str] = "", **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function creates a list with list data. New list's *Id* is returned.

        Args:
            listType: list's type ("persons" or "descriptors)
            listData: list data

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with created list id.

        """
        queries = {"type": listType}
        payload = RequestPayload.buildJson(body={"list_data": listData})
        return self.makeAuthRequest(
            url=f"{self.baseUri}/storage/lists", method="POST", body=payload, queryParams=queries, **kwargs
        )

    def getLists(
        self,
        page: Optional[int] = 1,
        pageSize: Optional[int] = 100,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function get all lists for the account.

        Args:
            page: page number, positive.
            pageSize: number of results per page, positive.

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with created lists.

        """
        queries = {"page": page, "page_size": pageSize}
        return self.makeAuthRequest(url=f"{self.baseUri}/storage/lists", method="GET", queryParams=queries, **kwargs)

    def deleteLists(self, lists: List[str], **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function removes lists.

        Args:
            lists: list of list ids.

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        payload = RequestPayload.buildJson(body={"lists": lists})
        return self.makeAuthRequest(url=f"{self.baseUri}/storage/lists", method="DELETE", body=payload, **kwargs)

    def getList(
        self, listId: str, page: Optional[int] = 1, pageSize: Optional[int] = 10, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function gets objects in the list.

        Args:
            listId: list *id*
            page: page number, positive.
            pageSize: number of results per page, positive.

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with list.

        """
        queries = {"page": page, "page_size": pageSize}
        return self.makeAuthRequest(
            url=f"{self.baseUri}/storage/lists/{listId}", method="GET", queryParams=queries, **kwargs
        )

    def patchListData(self, listId: str, listData: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function patches list data.

        Args:
            listId: list *id*
            listData: list data

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        payload = RequestPayload.buildJson(body={"list_data": listData})
        return self.makeAuthRequest(
            url=f"{self.baseUri}/storage/lists/{listId}", method="PATCH", body=payload, **kwargs
        )

    def deleteList(self, listId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function removes a list.

        Args:
            listId: list *id*

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        return self.makeAuthRequest(url=f"{self.baseUri}/storage/lists/{listId}", method="DELETE", **kwargs)

    def link(
        self,
        listId: str,
        descriptorIds: Optional[List[str]] = None,
        personIds: Optional[List[str]] = None,
        action: Optional[str] = "attach",
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Attach or detach descriptors or persons to list. Optional you can set, that list and descriptors must belong to
        account.

        Args:
            listId: list id
            action: attach or detach
            descriptorIds: descriptor ids
            personIds: person ids

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        data = {"action": action, "list_id": listId}
        if descriptorIds is not None:
            data["descriptor_ids"] = descriptorIds
        else:
            data["person_ids"] = personIds
        payload = RequestPayload.buildJson(body=data)
        return self.makeAuthRequest(url=f"{self.baseUri}/storage/linker", method="PATCH", body=payload, **kwargs)
