"""
Module realize ws mixin
"""

from contextlib import contextmanager
from numbers import Real
from typing import Optional, Type, Union

import aiohttp

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal

from ...common.ws_base import WSBase
from .base import BaseLunaBP3HttpClient


class WSMixin(WSBase):
    """WS Mixin to work with luna-backport3 websocket subscription."""

    # auth header; real method is defined above (BaseLunaBP3HttpClient)
    getAuthHeader: Type[BaseLunaBP3HttpClient.getAuthHeader]

    def wsConnect(  # pylint: disable=arguments-differ,arguments-renamed
        self,
        eventType: Optional[Literal["match", "extract"]] = None,
        observe: Optional[str] = None,
        gender: Optional[Literal["female", "male"]] = None,
        ageGt: Optional[float] = None,
        ageLt: Optional[float] = None,
        glassesGt: Optional[float] = None,
        glassesLt: Optional[float] = None,
        similarityGt: Optional[float] = None,
        matchingList: Optional[str] = None,
        asyncRequest: Optional[bool] = None,
        messageTimeout: Optional[Real] = None,
        login: Optional[str] = None,
        password: Optional[str] = None,
        token: Optional[str] = None,
    ) -> Union[aiohttp.client._WSRequestContextManager, contextmanager]:  # pylint: disable=W0212
        """
        Create ws-connection to luna-backport3

        Args:
            eventType: event type - 'match' or 'extract'
            observe: If you need to collect data only from special tokens, use this parameter in the following
                             format: %2C-separated list of tokens and "basic" for authorization with login/password pair
            gender: gender - 'female' or 'male'
            ageGt: lower limit of age to filter by
            ageLt: upper limit of age to filter by
            glassesGt: lower limit of glasses to filter by
            glassesLt: upper limit of glasses to filter by
            similarityGt: lower limit of similarity to filter by
            matchingList: list, works if event_type is not extract
            login: login for basic auth
            password: password for basic auth
            token: token for X-Auth-Token header
            asyncRequest: execution in asynchronous mode, disabled by default
            messageTimeout: message receive timeout (time between start waiting message and message appearance)
        Returns:
            context manager with connection to luna-backport3
        """
        queryParams = {
            "event_type": eventType,
            "observe": observe,
            "gender": gender,
            "age__lt": ageLt,
            "age__gt": ageGt,
            "glasses__gt": glassesGt,
            "glasses__lt": glassesLt,
            "similarity__gt": similarityGt,
            "list": matchingList,
        }

        headers = self.getAuthHeader(email=login, password=password, token=token)
        return super().wsConnect(
            headers=headers, queryParams=queryParams, messageTimeout=messageTimeout, asyncRequest=asyncRequest
        )
