"""
Module implements classes for standard structures.
"""

from typing import List, Optional

from luna3.common.http_objs import _ObjectFilters


class FaceFilters(_ObjectFilters):
    """
    Face filters

    Class Attributes:
        objectsType: current objects type

    Args:
        listId: id of list to which faces linked to
        createTimeLt: upper excluding bound for object create_time
        createTimeGte: lower including bound for object create_time
    """

    objectsType = "faces"

    def __init__(
        self, listId: Optional[str] = None, createTimeLt: Optional[str] = None, createTimeGte: Optional[str] = None
    ):
        super().__init__()
        self.filters.update(
            {
                "list_id": listId,
                "create_time__lt": createTimeLt,
                "create_time__gte": createTimeGte,
            }
        )


class EventFilters(_ObjectFilters):
    """
    Event filters without account id.

    Class Attributes:
        objectsType: current objects type

    Args:
        createTimeLt: upper excluding bound for object create_time
        createTimeGte: lower including bound for object create_time
        sources: sources filter
        eventsIds: event ids
        handlerIds: handler ids
        topSimilarFaceList: top similar face list
        topSimilarFaceIds: top similar face ids
        topSimilarFaceSimilarityGte: top similar face similarity lower including bound
        topSimilarFaceSimilarityLt: top similar face similarity upper excluding bound
        ageLt: age lower including bound
        ageGte: age upper excluding bound
        gender: gender filter 0 - man; 1 - woman
        emotions: emotions filter (from 1 to 7)
        ethnicGroups: ethnic group filter(from 1 to 4)
        faceIds: face ids
        userData: user data filter
        tags: tags filter
    """

    objectsType = "events"

    def __init__(
        self,
        createTimeLt: Optional[str] = None,
        createTimeGte: Optional[str] = None,
        sources: Optional[List[str]] = None,
        eventsIds: Optional[List[str]] = None,
        handlerIds: Optional[List[str]] = None,
        topSimilarFaceList: Optional[str] = None,
        topSimilarFaceIds: Optional[List[str]] = None,
        topSimilarFaceSimilarityGte: Optional[float] = None,
        topSimilarFaceSimilarityLt: Optional[float] = None,
        ageLt: Optional[int] = None,
        ageGte: Optional[int] = None,
        gender: Optional[int] = None,
        emotions: Optional[List[int]] = None,
        ethnicGroups: Optional[List[int]] = None,
        faceIds: Optional[List[str]] = None,
        userData: Optional[str] = None,
        tags: Optional[List[str]] = None,
    ):
        super().__init__()
        self.filters.update(
            {
                "create_time__lt": createTimeLt,
                "create_time__gte": createTimeGte,
                "sources": sources,
                "event_ids": eventsIds,
                "handler_ids": handlerIds,
                "top_similar_face_list": topSimilarFaceList,
                "top_similar_face_ids": topSimilarFaceIds,
                "top_similar_face_similarity__gte": topSimilarFaceSimilarityGte,
                "top_similar_face_similarity__lt": topSimilarFaceSimilarityLt,
                "age__lt": ageLt,
                "age__gte": ageGte,
                "gender": gender,
                "emotions": emotions,
                "ethnic_groups": ethnicGroups,
                "face_ids": faceIds,
                "user_data": userData,
                "tags": tags,
            }
        )
