"""
Luna-streams client module
"""

from numbers import Real
from typing import Any, Awaitable, Dict, List, Optional, Union

import aiohttp

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal

from ..common.base_api import BaseHTTPAPI
from ..common.luna_response import LunaResponse
from .http_objs import EventHandler, Location, QueueFilters, Restart, StreamFeedback


class StreamsApi(BaseHTTPAPI):
    """
    Class for request to luna-streams.

    """

    def __init__(
        self,
        origin: Optional[str] = "http://127.0.0.1:5160",
        api: Optional[int] = 1,
        asyncRequest: Optional[bool] = False,
        lunaRequestId: Optional[str] = None,
        totalTimeout: Real = 60,
        connectTimeout: Optional[Real] = None,
        sockReadTimeout: Optional[Real] = None,
        sockConnectTimeout: Optional[Real] = None,
        *,
        session: aiohttp.ClientSession = None,
    ):
        super().__init__(
            origin=origin,
            api=api,
            asyncRequest=asyncRequest,
            lunaRequestId=lunaRequestId,
            totalTimeout=totalTimeout,
            connectTimeout=connectTimeout,
            sockReadTimeout=sockReadTimeout,
            sockConnectTimeout=sockConnectTimeout,
            session=session,
        )

    @staticmethod
    def _createStreamGeneralBodyBuilder(
        *,
        accountId: str,
        name: Optional[str] = None,
        description: Optional[str] = None,
        streamType: Literal["udp", "tcp", "videofile", "images"],
        reference: str,
        mask: Optional[str] = None,
        endless: Optional[bool] = None,
        roi: Optional[Union[List[int], Dict]] = None,
        rotation: Optional[Literal[0, 90, 180, 270]] = None,
        location: Optional[Location] = None,
        restart: Optional[Restart] = None,
        status: Optional[Literal["pending", "pause"]] = None,
        groupName: Optional[str] = None,
        groupId: Optional[str] = None,
    ) -> dict:
        """
        Build template body for stream creation request.

        Args:
            name: stream name
            description: stream description
            streamType: stream type
            reference: stream URL
            mask: mask (for images)
            endless: if stream is endless (for tcp and udp streams only)
            roi: region of interest of camera frame (left, top, width, height), by default [0 0 0 0] - full frame
            rotation: angle of camera frame rotation (0, 90, 180 or 270)
            location: location that stream belongs
            restart: autorestart configuration
            status: stream processing initial status
            groupName: group name
            groupId: group identifier

        Returns:
            request body as dict.

        """
        body = {
            "account_id": accountId,
            "data": {"type": streamType, "reference": reference},
        }
        if name:
            body["name"] = name
        if description:
            body["description"] = description
        if mask:
            body["data"]["mask"] = mask
        if endless is not None:
            body["data"]["endless"] = endless
        if roi:
            body["data"]["roi"] = roi
        if rotation:
            body["data"]["rotation"] = rotation
        if restart:
            body["autorestart"] = restart.asDict()
        if location:
            body["location"] = location.asDict()
        if status:
            body["status"] = status
        if groupName:
            body["group_name"] = groupName
        if groupId:
            body["group_id"] = groupId

        return body

    def _createStreamV1BodyBuilder(
        self,
        *,
        accountId: str,
        name: Optional[str] = None,
        description: Optional[str] = None,
        streamType: Literal["udp", "tcp", "videofile", "images"],
        reference: str,
        mask: Optional[str] = None,
        endless: Optional[bool] = None,
        roi: Optional[Union[List[int], Dict]] = None,
        droi: Optional[Union[List[int], Dict]] = None,
        rotation: Optional[Literal[0, 90, 180, 270]] = None,
        preferredProgramStreamFrameWidth: Optional[int] = None,
        eventHandler: EventHandler,
        policies: Optional[Dict[str, Any]] = None,
        location: Optional[Location] = None,
        restart: Optional[Restart] = None,
        status: Optional[Literal["pending", "pause"]] = None,
        groupName: Optional[str] = None,
        groupId: Optional[str] = None,
    ) -> dict:
        """
        Build body for stream creation request.

        Args:
            name: stream name
            description: stream description
            streamType: stream type
            reference: stream URL
            mask: mask (for images)
            endless: if stream is endless (for tcp and udp streams only)
            roi: region of interest of camera frame (left, top, width, height), by default [0 0 0 0] - full frame
            droi: region of interest for detection considered as bestshot, by default [0 0 0 0] - full frame
            rotation: angle of camera frame rotation (0, 90, 180 or 270)
            preferredProgramStreamFrameWidth: frame width of the preferred program stream
            eventHandler: stream event handler configuration
            policies: stream processing policies
            location: location that stream belongs
            restart: autorestart configuration
            status: stream processing initial status
            groupName: group name
            groupId: group identifier

        Returns:
            request body as dict.

        """
        body = self._createStreamGeneralBodyBuilder(
            accountId=accountId,
            name=name,
            description=description,
            streamType=streamType,
            reference=reference,
            mask=mask,
            endless=endless,
            roi=roi,
            rotation=rotation,
            restart=restart,
            location=location,
            status=status,
            groupName=groupName,
            groupId=groupId,
        )
        if droi:
            body["data"]["droi"] = droi
        if preferredProgramStreamFrameWidth:
            body["data"]["preferred_program_stream_frame_width"] = preferredProgramStreamFrameWidth
        if policies:
            body["policies"] = policies
        body["event_handler"] = eventHandler.asDict()

        return body

    def _createStreamV2BodyBuilder(
        self,
        *,
        accountId: str,
        name: Optional[str] = None,
        description: Optional[str] = None,
        streamType: Literal["udp", "tcp", "videofile", "images"],
        reference: str,
        mask: Optional[str] = None,
        endless: Optional[bool] = None,
        roi: Optional[Union[List[int], Dict]] = None,
        rotation: Optional[Literal[0, 90, 180, 270]] = None,
        location: Optional[Location] = None,
        restart: Optional[Restart] = None,
        status: Optional[Literal["pending", "pause"]] = None,
        analytics: Optional[List[Dict]] = None,
        frameProcessingMode: Literal["auto", "full", "scale"] = None,
        realTimeModeFps: Optional[int] = None,
        ffmpegThreadsNumber: Optional[int] = None,
        groupName: Optional[str] = None,
        groupId: Optional[str] = None,
    ) -> dict:
        """
        Build body for stream creation request.

        Args:
            name: stream name
            description: stream description
            streamType: stream type
            reference: stream URL
            mask: mask (for images)
            endless: if stream is endless (for tcp and udp streams only)
            roi: region of interest of camera frame (left, top, width, height), by default [0 0 0 0] - full frame
            rotation: angle of camera frame rotation (0, 90, 180 or 270)
            location: location that stream belongs
            restart: autorestart configuration
            status: stream processing initial status
            analytics: stream analytics
            realTimeModeFps: real time mode fps
            frameProcessingMode: frame processing mode
            ffmpegThreadsNumber: ffmpeg threads number
            groupName: group name
            groupId: group identifier

        Returns:
            request body as dict.

        """
        body = self._createStreamGeneralBodyBuilder(
            accountId=accountId,
            name=name,
            description=description,
            streamType=streamType,
            reference=reference,
            mask=mask,
            endless=endless,
            roi=roi,
            rotation=rotation,
            restart=restart,
            location=location,
            status=status,
            groupName=groupName,
            groupId=groupId,
        )
        if analytics:
            body["data"]["analytics"] = analytics
        if frameProcessingMode:
            body["data"]["frame_processing_mode"] = frameProcessingMode
        if realTimeModeFps:
            body["data"]["real_time_mode_fps"] = realTimeModeFps
        if ffmpegThreadsNumber:
            body["data"]["ffmpeg_threads_number"] = ffmpegThreadsNumber

        return body

    def createStream(
        self,
        *,
        accountId: str,
        name: Optional[str] = None,
        description: Optional[str] = None,
        streamType: Literal["udp", "tcp", "videofile", "images"],
        reference: str,
        mask: Optional[str] = None,
        endless: Optional[bool] = None,
        roi: Optional[Union[List[int], Dict]] = None,
        droi: Optional[Union[List[int], Dict]] = None,
        rotation: Optional[Literal[0, 90, 180, 270]] = None,
        preferredProgramStreamFrameWidth: Optional[int] = None,
        eventHandler: Optional[EventHandler] = None,
        policies: Optional[Dict[str, Any]] = None,
        location: Optional[Location] = None,
        restart: Optional[Restart] = None,
        status: Optional[Literal["pending", "pause"]] = None,
        analytics: Optional[List[Dict]] = None,
        frameProcessingMode: Literal["auto", "full", "scale"] = None,
        realTimeModeFps: Optional[int] = None,
        ffmpegThreadsNumber: Optional[int] = None,
        groupName: Optional[str] = None,
        groupId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create new stream in luna-streams
        Args:
            accountId: stream account id
            name: stream name
            description: stream description
            streamType: stream type
            reference: stream URL
            mask: mask (for images)
            endless: if stream is endless (for tcp and udp streams only)
            roi: region of interest of camera frame (left, top, width, height), by default [0 0 0 0] - full frame
            droi: region of interest for detection considered as bestshot, by default [0 0 0 0] - full frame
            rotation: angle of camera frame rotation (0, 90, 180 or 270)
            preferredProgramStreamFrameWidth: frame width of the preferred program stream
            eventHandler: stream event handler configuration
            policies: stream processing policies
            location: location that stream belongs
            restart: autorestart configuration
            status: stream processing initial status
            analytics: stream analytics
            realTimeModeFps: real time mode fps
            frameProcessingMode: frame processing mode
            ffmpegThreadsNumber: ffmpeg threads number
            groupName: group name
            groupId: group identifier
        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with created stream id & version will be returned
        """
        if self.api == 1:
            if eventHandler is None:
                raise RuntimeError("Event handler must be set for streams V1 api")
            body = self._createStreamV1BodyBuilder(
                accountId=accountId,
                name=name,
                description=description,
                streamType=streamType,
                reference=reference,
                mask=mask,
                endless=endless,
                roi=roi,
                droi=droi,
                rotation=rotation,
                preferredProgramStreamFrameWidth=preferredProgramStreamFrameWidth,
                eventHandler=eventHandler,
                policies=policies,
                location=location,
                restart=restart,
                status=status,
                groupName=groupName,
                groupId=groupId,
            )
            return self.makeRequest(f"{self.baseUri}/streams", "POST", body=body, **kwargs)
        if self.api == 2:
            body = self._createStreamV2BodyBuilder(
                accountId=accountId,
                name=name,
                description=description,
                streamType=streamType,
                reference=reference,
                mask=mask,
                endless=endless,
                roi=roi,
                rotation=rotation,
                location=location,
                restart=restart,
                status=status,
                analytics=analytics,
                realTimeModeFps=realTimeModeFps,
                ffmpegThreadsNumber=ffmpegThreadsNumber,
                frameProcessingMode=frameProcessingMode,
                groupName=groupName,
                groupId=groupId,
            )
            return self.makeRequest(f"{self.baseUri}/streams", "POST", body=body, **kwargs)
        raise RuntimeError("Unsupported streams API version")

    def replaceStream(
        self,
        *,
        streamId: str,
        accountId: str,
        name: Optional[str] = None,
        description: Optional[str] = None,
        streamType: Literal["udp", "tcp", "videofile", "images"],
        reference: str,
        mask: Optional[str] = None,
        endless: Optional[bool] = None,
        roi: Optional[List[int]] = None,
        droi: Optional[List[int]] = None,
        rotation: Optional[Literal[0, 90, 180, 270]] = None,
        preferredProgramStreamFrameWidth: Optional[int] = None,
        eventHandler: Optional[EventHandler] = None,
        policies: Optional[Dict[str, Any]] = None,
        location: Optional[Location] = None,
        restart: Optional[Restart] = None,
        status: Optional[Literal["pending", "pause"]] = None,
        analytics: Optional[List[Dict]] = None,
        frameProcessingMode: Literal["auto", "full", "scale"] = None,
        realTimeModeFps: Optional[int] = None,
        ffmpegThreadsNumber: Optional[int] = None,
        groupName: Optional[str] = None,
        groupId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Replace an existent stream in luna-streams

        Args:
            streamId: id to put stream by
            accountId: stream account id
            name: stream name
            description: stream description
            streamType: stream type
            reference: stream URL
            mask: mask (for images)
            endless: if stream is endless (for tcp and udp streams only)
            roi: region of interest of camera frame (left, top, width, height), by default [0 0 0 0] - full frame
            droi: region of interest for detection considered as bestshot, by default [0 0 0 0] - full frame
            rotation: angle of camera frame rotation (0, 90, 180 or 270)
            preferredProgramStreamFrameWidth: frame width of the preferred program stream
            eventHandler: stream event handler configuration
            policies: stream processing policies
            location: location that stream belongs
            restart: autorestart configuration
            status: stream processing initial status
            analytics: analytics,
            realTimeModeFps: realTimeModeFps,
            ffmpegThreadsNumber: ffmpegThreadsNumber,
            frameProcessingMode: frameProcessingMode,
            groupName: group name
            groupId: group identifier

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with created stream id & version will be returned.

        """
        if self.api == 1:
            if eventHandler is None:
                raise RuntimeError("Event handler must be set for streams V1 api")
            body = self._createStreamV1BodyBuilder(
                accountId=accountId,
                name=name,
                description=description,
                streamType=streamType,
                reference=reference,
                mask=mask,
                endless=endless,
                roi=roi,
                droi=droi,
                rotation=rotation,
                preferredProgramStreamFrameWidth=preferredProgramStreamFrameWidth,
                eventHandler=eventHandler,
                policies=policies,
                location=location,
                restart=restart,
                status=status,
                groupName=groupName,
                groupId=groupId,
            )
            return self.makeRequest(f"{self.baseUri}/streams/{streamId}", "PUT", body=body, **kwargs)
        if self.api == 2:
            body = self._createStreamV2BodyBuilder(
                accountId=accountId,
                name=name,
                description=description,
                streamType=streamType,
                reference=reference,
                mask=mask,
                endless=endless,
                roi=roi,
                rotation=rotation,
                location=location,
                restart=restart,
                status=status,
                analytics=analytics,
                realTimeModeFps=realTimeModeFps,
                ffmpegThreadsNumber=ffmpegThreadsNumber,
                frameProcessingMode=frameProcessingMode,
                groupName=groupName,
                groupId=groupId,
            )
            return self.makeRequest(f"{self.baseUri}/streams/{streamId}", "PUT", body=body, **kwargs)
        raise RuntimeError("Unsupported streams API version")

    def getStreams(
        self,
        accountId: Optional[str] = None,
        streamIds: Optional[List[str]] = None,
        streamIdGte: Optional[str] = None,
        streamIdLt: Optional[str] = None,
        streamNames: Optional[List[str]] = None,
        reference: Optional[str] = None,
        statuses: Optional[List[str]] = None,
        createTimeGte: Optional[str] = None,
        createTimeLt: Optional[str] = None,
        groupName: Optional[str] = None,
        order: Optional[Literal["asc", "desc"]] = None,
        page: Optional[int] = None,
        pageSize: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get stream by filters.

        Args:
            accountId: account id
            streamIds: stream ids
            streamIdGte: stream id lower included bound
            streamIdLt: stream id upper excluded bound
            streamNames: stream names
            reference: stream reference
            statuses: stream current status values
            createTimeGte: stream create time lower including bound
            createTimeLt: stream create time upper excluding bound
            groupName: group name
            order: sorting order
            page: page number
            pageSize: page size

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with streams will be returned.

        """
        queries = {
            "account_id": accountId,
            "stream_ids": streamIds,
            "stream_id__gte": streamIdGte,
            "stream_id__lt": streamIdLt,
            "names": streamNames,
            "reference": reference,
            "statuses": statuses,
            "create_time__gte": createTimeGte,
            "create_time__lt": createTimeLt,
            "group_name": groupName,
            "order": order,
            "page": page,
            "page_size": pageSize,
        }
        return self.makeRequest(f"{self.baseUri}/streams", "GET", queryParams=queries, **kwargs)

    def countStreams(
        self,
        accountId: Optional[str] = None,
        streamIds: Optional[List[str]] = None,
        streamNames: Optional[List[str]] = None,
        reference: Optional[str] = None,
        statuses: Optional[List[str]] = None,
        createTimeGte: Optional[str] = None,
        createTimeLt: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get stream by filters.

        Args:
            accountId: account id
            streamIds: stream ids
            streamNames: stream names
            reference: stream reference
            statuses: stream current status values
            createTimeGte: stream create time lower including bound
            createTimeLt: stream create time upper excluding bound

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with streams will be returned.

        """
        queries = {
            "account_id": accountId,
            "stream_ids": streamIds,
            "names": streamNames,
            "reference": reference,
            "statuses": statuses,
            "create_time__gte": createTimeGte,
            "create_time__lt": createTimeLt,
        }
        return self.makeRequest(f"{self.baseUri}/streams/count", "GET", queryParams=queries, **kwargs)

    def getStream(
        self, streamId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get stream by id.

        Args:
            streamId: stream id
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with stream will be returned.

        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/streams/{streamId}", "GET", queryParams=queries, **kwargs)

    def patchStream(
        self,
        streamId: str,
        accountId: Optional[str] = None,
        description: Optional[str] = None,
        status: Optional[Literal["pending", "pause", "cancel"]] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Patch stream by id.

        Args:
            streamId: stream id
            accountId: account id
            description: new stream desription
            status: new stream status

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        body = {}
        if description is not None:
            body["description"] = description
        if status is not None:
            body["status"] = status

        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/streams/{streamId}", "PATCH", body=body, queryParams=queries, **kwargs)

    def deleteStream(
        self, streamId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete stream by id.

        Args:
            streamId: stream id
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/streams/{streamId}", "DELETE", queryParams=queries, **kwargs)

    def getStreamProcessingQueue(self, limit: int, filters: Optional[QueueFilters] = None, **kwargs) -> LunaResponse:
        """
        Get streams from the processing queue.

        Args:
            limit: number of streams to obtain
            filters: stream filters

        Returns:
            class:`~.LunaResponse` with streams
        """
        queries = {"limit": limit}
        if filters is not None:
            queries.update(filters.asDict())
        return self.makeRequest(f"{self.baseUri}/streams/processing/queue", "GET", queryParams=queries, **kwargs)

    def postStreamProcessingQueue(self, limit: int, filters: QueueFilters, **kwargs) -> LunaResponse:
        """
        Pull streams from the processing queue.

        Args:
            limit: number of streams to obtain
            filters: stream filters

        Returns:
            class:`~.LunaResponse` with streams
        """
        data = {"filters": filters.asDict(), "limit": limit}
        return self.makeRequest(f"{self.baseUri}/streams/processing/queue", "POST", body=data, **kwargs)

    def postFeedback(self, streams: List[StreamFeedback], **kwargs) -> LunaResponse:
        """
        Post streams feedback.

        Args:
            streams: streams to feedback

        Returns:
            class:`~.LunaResponse` with streams
        """
        data = {"streams": [stream.asDict() for stream in streams]}
        return self.makeRequest(f"{self.baseUri}/streams/processing/feedback", "POST", body=data, **kwargs)

    def getStreamsLogs(
        self,
        streamIds: Optional[List[str]] = None,
        statuses: Optional[List[str]] = None,
        targets: Optional[List[str]] = None,
        logTimeGte: Optional[str] = None,
        logTimeLt: Optional[str] = None,
        accountId: Optional[str] = None,
        page: Optional[int] = None,
        pageSize: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get stream logs by filters.

        Args:
            accountId: account id
            streamIds: stream ids
            statuses: stream current status values
            targets: targets
            logTimeGte: stream log record time lower including bound
            logTimeLt: stream log record time upper excluding bound
            page: page number
            pageSize: page size

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with streams will be returned.
        """
        queries = {
            "account_id": accountId,
            "stream_ids": streamIds,
            "statuses": statuses,
            "targets": targets,
            "log_time__gte": logTimeGte,
            "log_time__lt": logTimeLt,
            "page": page,
            "page_size": pageSize,
        }
        return self.makeRequest(f"{self.baseUri}/streams/logs", "GET", queryParams=queries, **kwargs)

    def deleteStreamsLogs(
        self,
        logTimeLt: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete streams logs by time filter excluding last log entry.

        Args:
            logTimeLt: stream log record time upper excluding bound

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with streams will be returned.
        """
        queries = {
            "log_time__lt": logTimeLt,
        }
        return self.makeRequest(f"{self.baseUri}/streams/logs", "DELETE", queryParams=queries, **kwargs)

    def deleteStreams(
        self,
        streamIds: Optional[List[str]] = None,
        streamNames: Optional[List[str]] = None,
        accountId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete streams by filters (OR).

        Args:
            streamIds: stream ids
            streamNames: names
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        body = {
            "stream_ids": streamIds,
            "names": streamNames,
        }
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/streams", "DELETE", body=body, queryParams=queries, **kwargs)

    def createGroup(
        self, groupName: str, accountId: str, description: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create new stream group.

        Args:
            accountId: account id
            groupName: group name
            description: group description

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with created group name and id will be returned.
        """
        body = {"group_name": groupName, "account_id": accountId}
        if description is not None:
            body["description"] = description
        return self.makeRequest(f"{self.baseUri}/groups", "POST", body=body, **kwargs)

    def getGroups(
        self,
        groupNames: Optional[List[str]] = None,
        groupIds: Optional[List[str]] = None,
        accountId: Optional[str] = None,
        page: Optional[int] = None,
        pageSize: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get groups by filters.

        Args:
            accountId: account id
            groupNames: group names
            groupIds: group ids
            page: page number
            pageSize: page size

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with groups will be returned.
        """
        queries = {
            "account_id": accountId,
            "group_ids": groupIds,
            "group_names": groupNames,
            "page": page,
            "page_size": pageSize,
        }
        return self.makeRequest(f"{self.baseUri}/groups", "GET", queryParams=queries, **kwargs)

    def countGroups(
        self,
        groupNames: Optional[List[str]] = None,
        groupIds: Optional[List[str]] = None,
        accountId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get count of groups by filter.

        Args:
            accountId: account id
            groupNames: group names
            groupIds: group ids

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with count of groups will be returned.
        """
        queries = {"account_id": accountId, "group_names": groupNames, "group_ids": groupIds}
        return self.makeRequest(f"{self.baseUri}/groups/count", "GET", queryParams=queries, **kwargs)

    def getGroup(
        self, groupId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get group.

        Args:
            groupId: group id
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with group will be returned.
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/groups/{groupId}", "GET", queryParams=queries, **kwargs)

    def patchGroup(
        self, groupId: str, description: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Update group description.

        Args:
            groupId: group id
            description: group description
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        queries = {"account_id": accountId}
        body = {"description": description}
        return self.makeRequest(f"{self.baseUri}/groups/{groupId}", "PATCH", queryParams=queries, body=body, **kwargs)

    def deleteGroup(
        self, groupId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete group.

        Args:
            groupId: group id
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/groups/{groupId}", "DELETE", queryParams=queries, **kwargs)

    def link(
        self,
        groupId: str,
        streamIds: List[str],
        action: Literal["attach", "detach"],
        accountId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Attach or detach streams to group. Optional you can set, that group and streams must belong to account.

        Args:
            groupId: group identifier
            streamIds: stream ids
            action: attach or detach
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        queries = {"account_id": accountId}
        data = {"action": action, "group_id": groupId, "stream_ids": streamIds}
        return self.makeRequest(f"{self.baseUri}/linker", "PATCH", queryParams=queries, body=data, **kwargs)

    def previewLive(
        self, streamId: str, accountId: Optional[str] = None, stream: bool = False, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Preview live.

        Args:
            accountId: account id
            streamId: stream id
            stream: whether to return request for streaming or consume all of the data in place

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        queries = {"account_id": accountId}
        return self.makeRequest(
            f"{self.baseUri}/streams/{streamId}/preview/handler/live",
            "GET",
            queryParams=queries,
            stream=stream,
            **kwargs,
        )

    def previewLastFrame(
        self, streamId: str, accountId: Optional[str] = None, stream: bool = False, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Preview last frame.

        Args:
            accountId: account id
            streamId: stream id
            stream: whether to return request for streaming or consume all of the data in place

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        queries = {"account_id": accountId}
        return self.makeRequest(
            f"{self.baseUri}/streams/{streamId}/preview/handler/frame",
            "GET",
            queryParams=queries,
            stream=stream,
            **kwargs,
        )
