"""
Module implements classes for standard structures.
"""

from typing import Optional

from ..common.http_objs import SubtaskCallback, TaskCallback

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal


class AEFaceFilters:
    """
    Face filters for additional extraction task

    Class Attributes:
        objectsType: current objects type

    Args:
        accountId: account id
        faceIdGte: upper excluding bound for object face id
        faceIdLt: lower including bound for object face id
    """

    objectsType = "faces"

    def __init__(
        self, accountId: Optional[str] = None, faceIdGte: Optional[str] = None, faceIdLt: Optional[str] = None
    ):
        self.accountId = accountId
        self.faceIdGte = faceIdGte
        self.faceIdLt = faceIdLt

    def asDict(self) -> dict:
        """Prepare dict representation."""
        filters = {"objects_type": self.objectsType}
        if self.accountId:
            filters["account_id"] = self.accountId
        if self.faceIdGte:
            filters["face_id__gte"] = self.faceIdGte
        if self.faceIdLt:
            filters["face_id__lt"] = self.faceIdLt
        return filters


class AEEventFilters:
    """
    Event filters for additional extraction task

    Class Attributes:
        objectsType: current objects type

    Args:
        accountId: account id
        handlerId: handler id
        eventIdGte: upper excluding bound for object event id
        eventIdLt: lower including bound for object event id
        insertTimeGte: event insert time upper excluding bound
        insertTimeLt: event insert time lower including bound
    """

    objectsType = "events"

    def __init__(
        self,
        accountId: Optional[str] = None,
        handlerId: Optional[str] = None,
        eventIdGte: Optional[str] = None,
        eventIdLt: Optional[str] = None,
        insertTimeGte: Optional[str] = None,
        insertTimeLt: Optional[str] = None,
    ):
        self.accountId = accountId
        self.handlerId = handlerId
        self.eventIdGte = eventIdGte
        self.eventIdLt = eventIdLt
        self.insertTimeGte = insertTimeGte
        self.insertTimeLt = insertTimeLt

    def asDict(self) -> dict:
        """Prepare dict representation."""
        filters = {"objects_type": self.objectsType}
        if self.accountId:
            filters["account_id"] = self.accountId
        if self.handlerId:
            filters["handler_id"] = self.handlerId
        if self.eventIdGte:
            filters["event_id__gte"] = self.eventIdGte
        if self.eventIdLt:
            filters["event_id__lt"] = self.eventIdLt
        if self.insertTimeGte:
            filters["insert_time__gte"] = self.insertTimeGte
        if self.insertTimeLt:
            filters["insert_time__lt"] = self.insertTimeLt
        return filters


class EstimatorZipSource:
    """
    Estimator task source - zip archive

    Class Attributes:
        sourceType: current task source - zip
    Args:
        url: link to archive
        prefix: file prefix
        postfix: file postfix
        password: archive password
        imageType: image type
        recursive: whether to estimate images from nested paths recursively
    """

    sourceType: str = "zip"

    def __init__(
        self,
        url: str,
        password: Optional[str] = None,
        imageType: Optional[int] = None,
        prefix: Optional[str] = None,
        postfix: Optional[str] = None,
        recursive: Optional[bool] = None,
    ):
        self.url = url
        self.password = password
        self.imageType = imageType
        self.prefix = prefix
        self.postfix = postfix
        self.recursive = recursive

    def asDict(self) -> dict:
        """Prepare dict representation."""
        res = dict(source_type=self.sourceType, reference=dict(url=self.url))
        if self.imageType is not None:
            res["image_type"] = self.imageType
        if self.password is not None:
            res["authorization"] = dict(password=self.password)
        if self.prefix is not None:
            res["reference"]["prefix"] = self.prefix
        if self.postfix is not None:
            res["reference"]["postfix"] = self.postfix
        if self.recursive is not None:
            res["recursive"] = self.recursive
        return res


class BaseEstimatorSource:
    """
    Base estimator task source

    Args:
        host: host
        port: port
        user: user
        password: user password
        path: path in server
        prefix: file prefix
        postfix: file postfix
        recursive: whether to estimate images from nested paths recursively
        imageType: image type
    """

    def __init__(
        self,
        host: str,
        path: Optional[str] = None,
        port: Optional[int] = None,
        user: Optional[str] = None,
        password: Optional[str] = None,
        prefix: Optional[str] = None,
        postfix: Optional[str] = None,
        recursive: Optional[bool] = None,
        imageType: Optional[int] = None,
    ):
        self.host = host
        self.port = port
        self.user = user
        self.password = password
        self.path = path
        self.prefix = prefix
        self.postfix = postfix
        self.recursive = recursive
        self.imageType = imageType

    def asDict(self) -> dict:
        """Prepare dict representation."""
        res = dict(reference=dict(host=self.host))
        if self.port is not None:
            res["reference"]["port"] = self.port
        if self.user is not None and self.password is not None:
            res["authorization"] = dict(user=self.user, password=self.password)
        if self.path is not None:
            res["reference"]["path"] = self.path
        if self.prefix is not None:
            res["reference"]["prefix"] = self.prefix
        if self.postfix is not None:
            res["reference"]["postfix"] = self.postfix
        if self.recursive is not None:
            res["recursive"] = self.recursive
        if self.imageType is not None:
            res["image_type"] = self.imageType
        return res


class EstimatorNetworkDiskSource:
    """
    Estimator task source - network disk

    Class Attributes:
        sourceType: current task source - network_disk
    Args:
        path: path to disk
        recursive: whether to estimate images from nested paths recursively
        followLinks: whether to follow file system links
        prefix: file prefix
        postfix: file postfix
        imageType: image type
    """

    sourceType: str = "network_disk"

    def __init__(
        self,
        path: str,
        recursive: Optional[bool] = None,
        followLinks: Optional[bool] = None,
        prefix: Optional[str] = None,
        postfix: Optional[str] = None,
        imageType: Optional[int] = None,
    ):
        self.path = path
        self.recursive = recursive
        self.followLinks = followLinks
        self.prefix = prefix
        self.postfix = postfix
        self.imageType = imageType

    def asDict(self) -> dict:
        """Prepare dict representation."""
        res = dict(source_type=self.sourceType, reference=dict(path=self.path))
        if self.recursive is not None:
            res["recursive"] = self.recursive
        if self.followLinks is not None:
            res["reference"]["follow_links"] = self.followLinks
        if self.imageType is not None:
            res["image_type"] = self.imageType
        if self.prefix is not None:
            res["reference"]["prefix"] = self.prefix
        if self.postfix is not None:
            res["reference"]["postfix"] = self.postfix
        return res


class EstimatorFTPSource(BaseEstimatorSource):
    """
    Estimator task source - FTP server

    Class Attributes:
        sourceType: current task source - ftp
    Args:
        host: FTP server host
        port: FTP server port
        user: FTP server user
        password: FTP server user password
        maxSessions: maximum amount of sessions on the FTP server
        path: path in FTP server
        prefix: file prefix
        postfix: file postfix
        recursive: whether to estimate images from nested paths recursively
        imageType: image type
    """

    sourceType: str = "ftp"

    def __init__(
        self,
        host: str,
        path: Optional[str] = None,
        port: Optional[int] = None,
        user: Optional[str] = None,
        password: Optional[str] = None,
        maxSessions: Optional[int] = None,
        prefix: Optional[str] = None,
        postfix: Optional[str] = None,
        recursive: Optional[bool] = None,
        imageType: Optional[int] = None,
    ):
        super().__init__(host, path, port, user, password, prefix, postfix, recursive, imageType)
        self.maxSessions = maxSessions

    def asDict(self) -> dict:
        """Prepare dict representation."""
        res = super().asDict()
        res["source_type"] = self.sourceType
        if self.maxSessions is not None:
            res["reference"]["max_sessions"] = self.maxSessions
        return res


class EstimatorS3Source:
    """
    Estimator task source - s3 bucket

    Class Attributes:
        sourceType: current task source - s3
    Args:
        endpoint: s3 endpoint
        prefix: file prefix
        bucketName: s3 bucket name or arn
        region: s3 region
        publicAccessKey: public access key
        secretAccessKey: secret access key
        authorizationSignatureVersion: one of 's3v2' or 's3v4'
        recursive: whether to estimate images from nested bucket paths recursively
        saveOrigin: whether to save image origins
        imageType: image type
        verifySSL: enable ssl certificate verification
    """

    sourceType: str = "s3"

    def __init__(
        self,
        bucketName: str,
        endpoint: Optional[str] = None,
        prefix: Optional[str] = None,
        region: Optional[str] = None,
        publicAccessKey: Optional[str] = None,
        secretAccessKey: Optional[str] = None,
        authorizationSignatureVersion: Optional[str] = None,
        recursive: Optional[bool] = None,
        saveOrigin: Optional[bool] = None,
        imageType: Optional[int] = None,
        verifySSL: Optional[bool] = None,
    ):
        self.endpoint = endpoint
        self.bucketName = bucketName
        self.region = region
        self.prefix = prefix
        self.publicAccessKey = publicAccessKey
        self.secretAccessKey = secretAccessKey
        self.authorizationSignatureVersion = authorizationSignatureVersion
        self.recursive = recursive
        self.saveOrigin = saveOrigin
        self.imageType = imageType
        self.verifySSL = verifySSL

    def asDict(self) -> dict:
        """Prepare dict representation."""
        res = dict(source_type=self.sourceType, reference=dict(bucket_name=self.bucketName))
        if self.endpoint is not None:
            res["reference"]["endpoint"] = self.endpoint
        if self.prefix is not None:
            res["reference"]["prefix"] = self.prefix
        if self.region is not None:
            res["reference"]["region"] = self.region
        if self.publicAccessKey is not None:
            res["authorization"] = dict(public_access_key=self.publicAccessKey, secret_access_key=self.secretAccessKey)
            if self.authorizationSignatureVersion is not None:
                res["authorization"]["signature_version"] = self.authorizationSignatureVersion
            if self.verifySSL is not None:
                res["authorization"]["verify_ssl"] = self.verifySSL
        if self.recursive is not None:
            res["recursive"] = self.recursive
        if self.saveOrigin is not None:
            res["save_origin"] = self.saveOrigin
        if self.imageType is not None:
            res["image_type"] = self.imageType
        return res


class EstimatorSambaSource(BaseEstimatorSource):
    """
    Estimator task source - Samba

    Class Attributes:
        sourceType: current task source - samba
    """

    sourceType: str = "samba"

    def asDict(self) -> dict:
        """Prepare dict representation."""
        res = super().asDict()
        res["source_type"] = self.sourceType
        return res


class TaskNotificationPolicy:
    """
    Task notification policy
    """

    def __init__(
        self, enable: Literal[0, 1], taskCallbacks: list[TaskCallback], subtaskCallbacks: list[SubtaskCallback]
    ):
        self.enable = enable
        self.taskCallbacks = taskCallbacks
        self.subtaskCallbacks = subtaskCallbacks

    def asDict(self) -> dict:
        """Prepare dict representation."""
        return {
            "enable": self.enable,
            "task_callbacks": [taskCallback.asDict() for taskCallback in self.taskCallbacks],
            "subtask_callbacks": [subtaskCallback.asDict() for subtaskCallback in self.subtaskCallbacks],
        }
